#include "config.h"
#include "utils.h"
#include "tree.h"
#include "convertTo.h"
#include "particledata.h"
#include "read_params.h"
#include "init.h"
#include "run.h"

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <stdexcept>

#ifdef _OPENMP
#include <omp.h>
#endif

using std::cout; 
using std::cin; 
using std::endl; 
using std::cerr; 
using std::string; 
using std::vector; 
using std::domain_error;
using std::ifstream; 

#ifdef _OPENMP
void set_opnemp(void); 
#endif


int main(int argc, char **argv) {
  
  try {
    string parameter_file; 
    if (argc < 2) 
    {
      throw domain_error("You should give the name of a parameter file"); 
    } else {
      ifstream in(argv[1]); 
      if (!in.is_open()) {
        cerr << "The name of the parameter file is " << argv[1] << endl; 
        throw domain_error("Can't open the parameter file."); 
      }
      read_params(in); 
      in.close(); 
    }

#ifdef _OPENMP
#ifndef OMP
    throw domain_error("Set -DOMP=? in the Makefile."); 
#endif
    set_opnemp(); 
#endif
    
    set_global(); 
    vector<ParticleData> particles; 
    initialize(particles); 
    run(0, particles); 

  } catch (std::bad_alloc& ) {
    cerr << "new threw badalloc" << endl; 
    abort(); 
  } catch (std::logic_error& e) {
    cerr << e.what() << endl; 
    abort(); 
  }


  return 0; 
}

#ifdef _OPENMP
void set_opnemp(void) {  
  cout << "default thread number = " << omp_get_max_threads() << endl; 
  omp_set_num_threads(OMP); 
  cout << "num_threads is = " << omp_get_max_threads() << endl; 
}
#endif
