#include "config.h"
#include "utils.h"
#include "particledata.h"
#include <iostream>
#include <vector>
#include <string> 
#include <sstream>
#include <fstream>
#include <stdexcept>


using std::vector; 
using std::string; 
using std::ofstream; 
using std::endl; 
using std::cerr; 
using std::domain_error; 

typedef vector<ParticleData>::size_type p_sz; 

void Output(const vector<ParticleData>& sph, const int snap_num) {

  ofstream out; 
  std::ostringstream file_name; 
  file_name << OutputFileBase << "_" << snap_num << ".txt"; 
  out.open(file_name.str().c_str()); 
  if (!out.is_open()) {
    cerr << "File: " << file_name.str() << " "; 
    throw domain_error("cannot be opened!"); 
  }
#ifdef ONE_D_SHOCKTUBE
  out << "# x v rho u P" << endl; 
#endif // closing of ONE_D_SHOCKTUBE

#ifdef TWO_D_KH
  out << "# x y vx vy rho u P" << endl; 
#endif

#ifdef THREE_D_SEDOV
  out << "# x y z vx vy vz rho u P r vr" << endl; 
#endif
  typedef vector<ParticleData>::size_type p_sz; 
  for (p_sz i = 0; i != sph.size(); ++i) {
    for(int dim = 0; dim != DIM; ++dim) 
      out << sph[i].GetPos(dim) << " "; 

    for (int dim = 0; dim != DIM; ++dim) 
      out << sph[i].GetVel(dim) << " ";  

    double rho = sph[i].GetDensity(); 
    out << rho << " "; 

    double pressure =  sph[i].GetEntropy() * pow(rho, Gamma); 
    // internal energy
    out << pressure/(Gamma_1 * rho) << " "; 
    // pressure 
    out << pressure ; 

#ifdef THREE_D_SEDOV
    double bhalf = 5.e-1 * BoxSize; 
    // radius from the box centre
    double x = sph[i].GetPos(0) - bhalf; 
    double y = sph[i].GetPos(1) - bhalf; 
    double z = sph[i].GetPos(2) - bhalf; 
    double r = sqrt( sq(x) + sq(y) + sq(z) ); 
    double vr = 0; 

    if (r > 0) 
      vr = (sph[i].GetVel(0) * x + sph[i].GetVel(1) * y + sph[i].GetVel(2) * z) / r; 

    out << " " << r << " " << vr;  

#endif

#ifdef USE_SMOOTHED_P
    out << " " << sph[i].GetSmoothedPressure(); 
#endif
    out << endl; 
  }

  out.close(); 
}
