#ifndef TREE_H
#define TREE_H
//-----------------------------------------------------
#include "config.h"
#include "particledata.h"
#include <vector>

const int nsub = (2 << (DIM - 1) ); 
//const int NodeCritNum = (nsub << 1); 
const int NodeCritNum = nsub; 
const int MaxNodeLevel = 31; 


class ParticleList {
  public: 
    typedef std::vector<ParticleData>::size_type pl_sz; 
    ParticleList(); 
    void PutIndex(pl_sz i) { index = i; } 
    pl_sz GetIndex() const { return index; }
  private:
    pl_sz index;  // point a corresponding particle; 
};  

class NodeInfo {
  public: 
    typedef std::vector<NodeInfo>::size_type nd_sz; 
    NodeInfo(); 
    // public functions
    
    real& Pos(const int dim) {
      return pos[dim]; 
    }
    void PutPos(const real x, const int dim) {
      pos[dim] = x; 
    }

    void PutPos(const real *x) {
      for (int dim = 0; dim != DIM; ++dim)
        pos[dim] = x[dim]; 
    }

    real GetPos(const int dim) const { return pos[dim]; }
    const real *GetPos() { return &pos[0]; }


    void PutWidth(const real w) { width = w; }  
    real GetWidth() const { return width; }

    void PutNumberOfLeaves(const std::vector<ParticleList>::size_type n) { number = n; }
    std::vector<ParticleList>::size_type GetNumberOfLeaves() const { return number; }

    void PutHsml(const double h) { max_hsml = h; }
    double GetHsml() const { return max_hsml; }
    
    void PutTopLeaf(const std::vector<ParticleList>::size_type n) { top_leaf = n; }
    std::vector<ParticleList>::size_type GetTopLeaf() const { return top_leaf; }


    void PutLevel(const int l) { level = l; }
    int GetLevel() const { return level; }

    void PutParent(const nd_sz p) { parent = p; }
    nd_sz GetParent() const { return parent; }

    void PutChild(const nd_sz c) { child = c; }
    nd_sz GetChild() const { return child; }

    void PutNext(const nd_sz n) { next = n; }
    nd_sz GetNext() const { return next; }

    void PutBrother(const nd_sz j) { brother = j; }
    nd_sz GetBrother() const { return brother; }

  private: 
    nd_sz next; 
    nd_sz parent; 
    nd_sz child; 
    nd_sz brother; 

    int level; 
    std::vector<ParticleList>::size_type number; 
    std::vector<ParticleList>::size_type top_leaf; 
    
    real pos[DIM]; 
    real width; 
    double max_hsml; 
}; 

void BuildTree(std::vector<NodeInfo>& tree, std::vector<ParticleList>& leaves, const std::vector<ParticleData>& sph); 
void UpdateHmax(std::vector<NodeInfo>& tree, const std::vector<ParticleList>& plist, const std::vector<ParticleData>& sph); 

//-----------------------------------------------------
#endif 
